"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var apply_events_1 = require("./apply-events");
var chai_1 = require("chai");
require("mocha");
describe('applyEvents()', function () {
    it('should add one tag', function () {
        var entries = [{ id: '1' }];
        var events = [
            {
                type: 'userAction',
                id: '5',
                date: '2022-02-22',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'addTag',
                        value: 'foo'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([{ id: '1', updated: '2022-02-22', tags: ['foo'], appliedEventIds: ['5'] }]);
        (0, chai_1.expect)(entries[0]).to.eql(result[0]);
    });
    it('should add one tag to tow entries with same id', function () {
        var entries = [{ id: '1' }, { id: '1' }];
        var events = [
            {
                type: 'userAction',
                id: '5',
                date: '2022-02-22',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'addTag',
                        value: 'foo'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([{ id: '1', updated: '2022-02-22', tags: ['foo'], appliedEventIds: ['5'] }, { id: '1', updated: '2022-02-22', tags: ['foo'], appliedEventIds: ['5'] }]);
        (0, chai_1.expect)(entries[0]).to.eql(result[0]);
    });
    it('should add two tags', function () {
        var entries = [{ id: '1', updated: '2022-03-06' }];
        var events = [
            {
                type: 'userAction',
                id: '6',
                date: '2022-02-22',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'addTag',
                        value: 'foo'
                    },
                    {
                        action: 'addTag',
                        value: 'bar'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([{ id: '1', updated: '2022-03-06', tags: ['foo', 'bar'], appliedEventIds: ['6'] }]);
        (0, chai_1.expect)(entries[0]).to.eql(result[0]);
    });
    it('should remove one tag', function () {
        var entries = [{ id: '1', updated: '2022-02-22', tags: ['foo', 'bar'] }];
        var events = [
            {
                type: 'userAction',
                id: '7',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'removeTag',
                        value: 'foo'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([{ id: '1', updated: '2022-02-22', tags: ['bar'], appliedEventIds: ['7'] }]);
        (0, chai_1.expect)(entries[0]).to.eql(result[0]);
    });
    it('should remove two tags', function () {
        var entries = [{ id: '1', updated: '2022-01-01', tags: ['foo', 'bar'] }];
        var events = [
            {
                type: 'userAction',
                id: '8',
                date: '2022-02-22',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'removeTag',
                        value: 'foo'
                    },
                    {
                        action: 'removeTag',
                        value: 'bar'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([{ id: '1', updated: '2022-02-22', tags: [], appliedEventIds: ['8'] }]);
        (0, chai_1.expect)(entries[0]).to.eql(result[0]);
    });
    it('should not apply event', function () {
        var entries = [{ id: '1', updated: '2022-01-01', tags: ['foo'], appliedEventIds: ['9'] }];
        var events = [
            {
                type: 'userAction',
                id: '9',
                date: '2022-02-22',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'removeTag',
                        value: 'foo'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([]);
        (0, chai_1.expect)(entries[0]).to.eql({ id: '1', updated: '2022-01-01', tags: ['foo'], appliedEventIds: ['9'] });
    });
    it('should not add existing tag', function () {
        var entries = [{ id: '1', updated: '2022-01-01', tags: ['foo'] }];
        var events = [
            {
                type: 'userAction',
                date: '2022-02-22',
                id: '10',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'addTag',
                        value: 'foo'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([]);
        (0, chai_1.expect)(entries[0]).to.eql({ id: '1', updated: '2022-01-01', tags: ['foo'], appliedEventIds: ['10'] });
    });
    it('should add and remove tag', function () {
        var entries = [{ id: '1' }];
        var events = [
            {
                type: 'userAction',
                id: '8',
                targetIds: ['1'],
                actions: [
                    {
                        action: 'addTag',
                        value: 'foo'
                    },
                    {
                        action: 'removeTag',
                        value: 'foo'
                    }
                ]
            }
        ];
        var result = (0, apply_events_1.applyEvents)(entries, events);
        (0, chai_1.expect)(result).to.eql([{ id: '1', tags: [], appliedEventIds: ['8'] }]);
        (0, chai_1.expect)(entries[0]).to.eql(result[0]);
    });
});
