"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readEvents = void 0;
var fs_1 = __importDefault(require("fs"));
var readline_1 = __importDefault(require("readline"));
var util_1 = require("util");
var log = require('@home-gallery/logger')('events.read');
var header_1 = require("./header");
var readEventsCb = function (filename, cb) {
    var isClosed = false;
    var header = false;
    var events = [];
    var reader;
    var lineNumber = 0;
    var readStream = fs_1.default.createReadStream(filename);
    readStream.on('error', function (err) {
        isClosed = true;
        cb(err);
        reader && reader.close();
    });
    var checkHeader = function (data) {
        if (!(0, header_1.isEventTypeCompatible)(data === null || data === void 0 ? void 0 : data.type)) {
            var err = new Error("Unknown events database header: '".concat(JSON.stringify(data).substring(0, 45), "...'. Expect '{\"type\":\"").concat(header_1.HeaderType, "\",...}'"));
            log.error(err, err.message);
            return readStream.destroy(err);
        }
        header = data;
    };
    reader = readline_1.default.createInterface({
        input: readStream,
        terminal: false
    });
    reader.on('line', function (line) {
        try {
            var data = JSON.parse(line);
            if (lineNumber++ > 0) {
                events.push(data);
            }
            else {
                return checkHeader(data);
            }
        }
        catch (e) {
            log.error(e, "Could not parse line ".concat(lineNumber, ": ").concat(line));
        }
    });
    reader.on('error', function (err) {
        if (isClosed) {
            return;
        }
        isClosed = true;
        cb(err);
        reader.close();
    });
    reader.on('close', function () {
        if (isClosed) {
            return;
        }
        else if (header) {
            header.data = events;
            cb(null, header);
        }
        else {
            cb(new Error("No event header found"));
        }
    });
};
exports.readEvents = (0, util_1.promisify)(readEventsCb);
