"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.htmlEncode = exports.stripSuffix = exports.stripPrefix = exports.pad3 = exports.pad2 = exports.leftPad = exports.toS = exports.notBlank = exports.blank = exports.isString = void 0;
var he_1 = require("he");
var Array_1 = require("./Array");
function isString(o) {
    return typeof o === "string";
}
exports.isString = isString;
var spaces = Array_1.times(10, function (i) { return Array_1.times(i, function () { return " "; }).join(""); });
var zeroes = Array_1.times(10, function (i) { return Array_1.times(i, function () { return "0"; }).join(""); });
function blank(s) {
    return s == null || String(s).trim().length === 0;
}
exports.blank = blank;
function notBlank(s) {
    return !blank(s);
}
exports.notBlank = notBlank;
function padding(padChar, count) {
    if (count <= 0)
        return "";
    return (padChar === "0" ? zeroes : spaces)[Math.floor(count)];
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function toS(s) {
    return s == null ? "" : String(s);
}
exports.toS = toS;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function leftPad(i, minLen, padChar) {
    if (i == null || isNaN(i))
        i = 0;
    if (i < 0) {
        return "-" + leftPad(-i, minLen - 1, padChar);
    }
    else {
        var s = String(i);
        return padding(padChar, minLen - s.length) + s;
    }
}
exports.leftPad = leftPad;
function pad2() {
    var numbers = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        numbers[_i] = arguments[_i];
    }
    return numbers.map(function (i) { return leftPad(i, 2, "0"); });
}
exports.pad2 = pad2;
function pad3() {
    var numbers = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        numbers[_i] = arguments[_i];
    }
    return numbers.map(function (i) { return leftPad(i, 3, "0"); });
}
exports.pad3 = pad3;
/**
 * NOTE: case insensitive
 */
function stripPrefix(s, prefix) {
    return toS(s).toLowerCase().startsWith(prefix.toLowerCase())
        ? s.slice(prefix.length)
        : s;
}
exports.stripPrefix = stripPrefix;
function stripSuffix(s, suffix) {
    var str = toS(s);
    return str.endsWith(suffix) ? str.slice(0, -suffix.length) : str;
}
exports.stripSuffix = stripSuffix;
function htmlEncode(s) {
    // `he` doesn't encode whitespaces (like newlines), but we need that:
    return he_1.encode(s, { decimal: true }).replace(/\s/g, function (m) {
        return m[0] === " " ? " " : "&#" + m[0].charCodeAt(0) + ";";
    });
}
exports.htmlEncode = htmlEncode;
//# sourceMappingURL=String.js.map